import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from matplotlib import rc
rc('text',usetex=True)
plt.rc('text.latex', preamble=r'\usepackage{amsmath} \usepackage{siunitx}')
def draw_arrow(ax,Xa,Xb,**opt):
	def get_key(key,default_value):
		return default_value if key not in opt else opt[key]
	arrowstyle = get_key('arrowstyle','-')
	xycoords = get_key('xycoords','data')
	if (any(elem in arrowstyle for elem in '<>')):
		head_length = get_key('head_length','0.8');
		head_width  = get_key('head_width' ,'0.4');
		arrowstyle+=',head_length = ' + str(head_length) + ',head_width = ' + str(head_width)
	return ax1.annotate(get_key('text',''),
	                   xytext = (Xa[0],Xa[1]),
	                   xy = (Xb[0],Xb[1]),
	                   xycoords = xycoords,
	                   textcoords = xycoords,
	                   zorder = get_key('zorder',-1),
	                   annotation_clip = get_key('annotation_clip',False),
	                   ha='center',
	                   va='center',
	                   arrowprops = dict(arrowstyle = arrowstyle,
	                                     shrinkA = get_key('shrinkA',0),
	                                     shrinkB = get_key('shrinkB',0),
	                                     linestyle = get_key('linestyle','-'),
	                                     linewidth = get_key('linewidth',1),
	                                     facecolor = get_key('color','black'),
	                                     edgecolor = get_key('color','black'),
	                                     clip_path = get_key('clip',None)))
def import_data(S='S1'):
	raw_data='Fadh.dat'
	cols=(0,1)
	if S=='S2':
		cols=(2,3)
	if S=='S3':
		cols=(4,5)
	Fa,Fa_over_R = np.loadtxt(raw_data,delimiter='\t',usecols=cols,unpack=True,skiprows=1,dtype=np.float64)
	return np.sort(Fa),np.sort(Fa_over_R)
def my_distribtion(x,bins=50,cumulative=True):
	#https://numpy.org/devdocs/reference/generated/numpy.histogram.html
	#https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.hist.html#matplotlib.pyplot.hist
	counts, bin_edges = np.histogram(x,bins=bins,density=False)
	bin_centers = (bin_edges[1:] + bin_edges[:-1])/2
	if cumulative:
		counts = np.cumsum(counts)
	return bin_centers,counts
def print_info(bins,counts,substrate=-1):
	y = bins
	x = counts
	xs_interp = [0.25,0.5,0.75]
	ys_interp = np.interp(xs_interp,x,y)
	q1,q2,q3 = ys_interp
	if substrate>0:
		print("S{}: F_median = {:.2f} IQR = {:.2f}".format(substrate,q2,q3-q1))
	else:
		print("F_median = {:.2f} IQR = {:.2f}".format(q,q3-q1))
fig = plt.figure(figsize=(10,5)) #figsize=(10,10)
fs = 14
lw = 1
hspace = 0.2
margin = 0.1
x0 = margin
y0 = margin
Lx0 = 0.5-hspace/2
Ly0 = 1.0-2*margin
ax0 = fig.add_axes([x0,y0,Lx0,Ly0])
x1 = 0.5+hspace/2
y1 = margin
Lx1 = 0.5-hspace/2-margin
Ly1 = 1.0-2*margin
ax1 = fig.add_axes([x1,y1,Lx1,Ly1])
plot_labels=['(a)','(b)']
for label,ax in zip(plot_labels,[ax0,ax1]):
	ax.xaxis.set_tick_params(which='major',length=5,direction='in',width=lw,pad=5)
	ax.yaxis.set_tick_params(which='major',length=5,direction='in',width=lw,pad=5)
	ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=lw,pad=5)
	ax.xaxis.set_ticks_position('both')
	ax.yaxis.set_ticks_position('both')
	for axis in ['top','bottom','left','right']:
		ax.spines[axis].set_linewidth(lw)
	ax.text(0.02,0.98,label,fontsize=fs,transform=ax.transAxes,ha='left',va='top')
labels=['S1','S2','S3']
colors=['C0','C1','C2']
#~~~~~~~~
# PLOT 1
#~~~~~~~~
ax0.set_xscale('log')
ax0.set_xlim(30,3000)
ax0.set_ylim(0,1.001)
xticks = [10,100,1000,3000]
yticks = [0,0.25,0.5,0.75,1.001]
xlabels = [r'${}$'.format(tick) for tick in xticks]
ylabels = [r'${:.2f}$'.format(tick) for tick in yticks]
ax0.set_xticks(xticks)
ax0.set_yticks(yticks)
ax0.set_xticklabels(xlabels,fontsize=fs)
ax0.set_yticklabels(ylabels,fontsize=fs)
ax0.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
bins = 500
for i,label in enumerate(labels):
	cumulative = True
	forces,_ = import_data(S=label)
	bin_centers, counts = my_distribtion(x=forces,bins=bins,cumulative=cumulative)
	counts = counts/counts[-1]
	bin_centers = np.append(bin_centers,3000)
	counts = np.append(counts,counts[-1])
	ax0.plot(bin_centers,counts,'-',label=label,color=colors[i])
	q1,q2,q3,q9 = np.percentile(forces,[25,50,75,90])
	print("S{}: F_mean = {:.2f} F_median = {:.2f} IQR = {:.2f} q9 = {:2f}".format(i+1,np.mean(forces),q2,q3-q1,q9))
	print_info(bin_centers,counts,substrate=i+1)
handles_lgd, labels_lgd = ax0.get_legend_handles_labels()
#sort=[0,1,2]
#labels = [labels[i] for i in sort]
#handles = [handles[i] for i in sort]
ax0.legend(handles_lgd,
          labels_lgd,
          ncol = 1,
          fontsize = fs,
          loc = 'lower right',
          #bbox_to_anchor=(0.77, 0.3),
          handlelength = 1,
          handletextpad = 0.2,
          columnspacing = 1,
          frameon = False)
ax0.annotate(r'$F_a \text{(nN)}$',
            xy = (0.50,-0.10),
            xycoords = 'axes fraction',
            ha = 'center',
            va = 'center',
            fontsize = fs,
            annotation_clip = False)
ax0.annotate('Cumulative frequency',
            xy = (-0.15,0.50),
            xycoords = 'axes fraction',
            ha = 'center',
            va = 'center',
            fontsize = fs,
            rotation = 90,
            annotation_clip = False)
#~~~~~~~~
# PLOT 2
#~~~~~~~~
box_positions=[0.5,0.95,1.5]
ax1.set_xlim(0.3,2.35)
ax1.set_ylim(0,1800)
yticks = [0,300,600,900,1200,1500,1800]
ylabels = [r'${}$'.format(tick) for tick in yticks]
#ax1.set_xticks(xticks)
ax1.set_yticks(yticks)
ax1.set_xticklabels(labels,fontsize=fs)
ax1.set_yticklabels(ylabels,fontsize=fs)
data = []
#ax1.set_yscale('log')
for i,label in enumerate(labels):
	forces,_ = import_data(S=label)
	data.append(forces)
	#Percentiles
	q1,q2,q3,q9 = np.percentile(forces,[25,50,75,90])
	ax1.plot([box_positions[i]],[q9],marker='_',markersize=8,markeredgewidth=2)
	#xs = [i+1,i+1]
	#ys = [q1,q3]
	#ax1.scatter(xs,ys)
	##median
	#xs = [i+1]
	#ys = [q2]
	#ax1.scatter(xs,ys)
	##min/max
	#xs = [i+1,i+1]
	#ys = [forces.min(),forces.max()]
	#ax1.scatter(xs,ys)
	##mean
	#xs = [i+1]
	#ys = [forces.mean()]
	#ax1.scatter(xs,ys)
	##All data
	#xs = np.ones(forces.size)*(i+1)
	#ax1.scatter(xs,forces, alpha=0.4, color=colors[i])
x = box_positions[-1]+0.15
ax1.text(x,q1,r'$q_{25}$',fontsize = fs,va='center',color=colors[-1])
ax1.text(x,q2,r'Median ($q_{50}$)',fontsize = fs,va='center',color=colors[-1])
ax1.text(x,q3,r'$q_{75}$',fontsize = fs,va='center',color=colors[-1])
ax1.text(x,q9,r'$q_{90}$',fontsize = fs,va='center',color=colors[-1])
ax1.text(x,forces.mean(),'Mean',fontsize = fs,va='center',color=colors[-1])
#ax1.text(x,forces.min(),'Minimum',fontsize = fs,va='top',color=colors[-1])
ax1.text(x,forces.max(),'Maximum',fontsize = fs,va='center',color=colors[-1])
x = box_positions[-1]-0.20
draw_arrow(ax1,[x,q1],[x,q3],arrowstyle='<|-|>',head_width=0.3,head_length=0.6)
ax1.text(x-0.02,forces.mean(),'IQR',fontsize = fs,va='center',ha='right',color=colors[-1],rotation=90)
bplot = ax1.boxplot(data,
                    patch_artist = True,
                    showmeans = True,
                    labels = labels,
                    widths = 0.25,
                    positions=box_positions)
for patch,color in zip(bplot['boxes'],colors):
	patch.set_facecolor(matplotlib.colors.to_rgba(color, 0.5))
	#patch.set_edgecolor(matplotlib.colors.to_rgba(color, 0.5))
	#patch.set_alpha(0.4)
	patch.set_linewidth(0.)
for patch,color in zip(bplot['medians'],colors):
	patch.set_color(color)
	patch.set_linewidth(2.0)
for patch,color in zip(bplot['means'],colors):
	patch.set_marker('o')
	patch.set_markerfacecolor(color)
	patch.set_markeredgewidth(0)
	#patch.set_linewidth(2.0)
for i,patch in enumerate(bplot['caps']):
	i_corr = int(i/2) #0,1,2,3,4,5 -> 0,0,1,1,2,2
	patch.set_color(colors[i_corr])
	patch.set_linewidth(2.0)
for i,patch in enumerate(bplot['whiskers']):
	i_corr = int(i/2)
	patch.set_color(colors[i_corr])
	patch.set_linewidth(2.0)
ax1.annotate('Scenario',
            xy = (0.50,-0.10),
            xycoords = 'axes fraction',
            ha = 'center',
            va = 'center',
            fontsize = fs,
            annotation_clip = False)
ax1.annotate(r'$F_a \text{(nN)}$',
            xy = (-0.20,0.50),
            xycoords = 'axes fraction',
            ha = 'center',
            va = 'center',
            fontsize = fs,
            rotation = 90,
            annotation_clip = False)
plt.savefig('plot.pdf')
plt.close()
